# frozen_string_literal: true

module Rspeckled
module Helpers
module Filepaths
  def root_filepath
    @root_filepath ||= begin
                         current_directory = ::Dir.pwd

                         until ::Dir.exist?("#{current_directory}/spec")
                           current_directory += '/..'
                         end

                         ::File.expand_path(current_directory)
                       end
  end

  def fixture_filepath(filepath)
    "#{root_filepath}/spec/fixtures/#{filepath}"
  end

  def read_fixture(filepath)
    ::File.read(fixture_filepath(filepath))
  end

  def parse_fixture(filepath)
    contents = read_fixture(filepath)

    case filepath[/\.\w+\z/]
    when '.json'
      ::JSON.parse(contents)
    when '.eml'
      ::Mail.new(contents)
    else
      fail ::ArgumentError, "I don't know how to parse #{filepath}."
    end
  end
end
end
end
